@@ -289,7 +289,7 @@ type completionContext struct {
 // A Selection represents the cursor position and surrounding identifier.
 type Selection struct {
 	content string
-	cursor  token.Pos
+	cursor  token.Pos // relative to rng.TokFile
 	rng     span.Range
 }
 
@@ -297,12 +297,8 @@ func (p Selection) Content() string {
 	return p.content
 }
 
-func (p Selection) Start() token.Pos {
-	return p.rng.Start
-}
-
-func (p Selection) End() token.Pos {
-	return p.rng.End
+func (p Selection) Range() span.Range {
+	return p.rng
 }
 
 func (p Selection) Prefix() string {
@@ -693,7 +689,7 @@ func (c *completer) containingIdent(src []byte) *ast.Ident {
 
 // scanToken scans pgh's contents for the token containing pos.
 func (c *completer) scanToken(contents []byte) (token.Pos, token.Token, string) {
-	tok := c.snapshot.FileSet().File(c.pos)
+	tok := c.pkg.FileSet().File(c.pos)
 
 	var s scanner.Scanner
 	s.Init(tok, contents, nil, 0)
@@ -879,7 +875,7 @@ func (c *completer) populateCommentCompletions(ctx context.Context, comment *ast
 	}
 
 	// Using the comment position find the line after
-	file := c.snapshot.FileSet().File(comment.End())
+	file := c.pkg.FileSet().File(comment.End())
 	if file == nil {
 		return
 	}
@@ -1087,12 +1083,7 @@ func (c *completer) selector(ctx context.Context, sel *ast.SelectorExpr) error {
 	// Is sel a qualified identifier?
 	if id, ok := sel.X.(*ast.Ident); ok {
 		if pkgName, ok := c.pkg.GetTypesInfo().Uses[id].(*types.PkgName); ok {
-			var pkg source.Package
-			for _, imp := range c.pkg.Imports() {
-				if imp.PkgPath() == pkgName.Imported().Path() {
-					pkg = imp
-				}
-			}
+			pkg, _ := c.pkg.DirectDep(source.PackagePath(pkgName.Imported().Path()))
 			// If the package is not imported, try searching for unimported
 			// completions.
 			if pkg == nil && c.opts.unimported {
@@ -1140,7 +1131,7 @@ func (c *completer) unimportedMembers(ctx context.Context, id *ast.Ident) error
 		if pkg.GetTypes().Name() != id.Name {
 			continue
 		}
-		paths = append(paths, path)
+		paths = append(paths, string(path))
 	}
 
 	var relevances map[string]float64
@@ -1158,7 +1149,7 @@ func (c *completer) unimportedMembers(ctx context.Context, id *ast.Ident) error
 	})
 
 	for _, path := range paths {
-		pkg := known[path]
+		pkg := known[source.PackagePath(path)]
 		if pkg.GetTypes().Name() != id.Name {
 			continue
 		}
@@ -1182,7 +1173,8 @@ func (c *completer) unimportedMembers(ctx context.Context, id *ast.Ident) error
 	add := func(pkgExport imports.PackageExport) {
 		mu.Lock()
 		defer mu.Unlock()
-		if _, ok := known[pkgExport.Fix.StmtInfo.ImportPath]; ok {
+		// TODO(adonovan): what if the actual package has a vendor/ prefix?
+		if _, ok := known[source.PackagePath(pkgExport.Fix.StmtInfo.ImportPath)]; ok {
 			return // We got this one above.
 		}
 
@@ -1245,7 +1237,7 @@ func (c *completer) methodsAndFields(typ types.Type, addressable bool, imp *impo
 
 	if isStarTestingDotF(typ) && addressable {
 		// is that a sufficient test? (or is more care needed?)
-		if c.fuzz(typ, mset, imp, cb, c.snapshot.FileSet()) {
+		if c.fuzz(typ, mset, imp, cb, c.pkg.FileSet()) {
 			return
 		}
 	}
@@ -1330,7 +1322,7 @@ func (c *completer) lexical(ctx context.Context) error {
 					node = c.path[i-1]
 				}
 				if node != nil {
-					if resolved := resolveInvalid(c.snapshot.FileSet(), obj, node, c.pkg.GetTypesInfo()); resolved != nil {
+					if resolved := resolveInvalid(c.pkg.FileSet(), obj, node, c.pkg.GetTypesInfo()); resolved != nil {
 						obj = resolved
 					}
 				}
@@ -1379,7 +1371,8 @@ func (c *completer) lexical(ctx context.Context) error {
 
 				// Make sure the package name isn't already in use by another
 				// object, and that this file doesn't import the package yet.
-				if _, ok := seen[pkg.Name()]; !ok && pkg != c.pkg.GetTypes() && !alreadyImports(c.file, pkg.Path()) {
+				// TODO(adonovan): what if pkg.Path has vendor/ prefix?
+				if _, ok := seen[pkg.Name()]; !ok && pkg != c.pkg.GetTypes() && !alreadyImports(c.file, source.ImportPath(pkg.Path())) {
 					seen[pkg.Name()] = struct{}{}
 					obj := types.NewPkgName(0, nil, pkg.Name(), pkg)
 					imp := &importInfo{
@@ -1477,16 +1470,19 @@ func (c *completer) unimportedPackages(ctx context.Context, seen map[string]stru
 
 	count := 0
 
+	// TODO(adonovan): strength-reduce to a metadata query.
+	// All that's needed below is Package.{Name,Path}.
+	// Presumably that can be answered more thoroughly more quickly.
 	known, err := c.snapshot.CachedImportPaths(ctx)
 	if err != nil {
 		return err
 	}
-	var paths []string
+	var paths []string // actually PackagePaths
 	for path, pkg := range known {
 		if !strings.HasPrefix(pkg.GetTypes().Name(), prefix) {
 			continue
 		}
-		paths = append(paths, path)
+		paths = append(paths, string(path))
 	}
 
 	var relevances map[string]float64
@@ -1511,7 +1507,7 @@ func (c *completer) unimportedPackages(ctx context.Context, seen map[string]stru
 	})
 
 	for _, path := range paths {
-		pkg := known[path]
+		pkg := known[source.PackagePath(path)]
 		if _, ok := seen[pkg.GetTypes().Name()]; ok {
 			continue
 		}
@@ -1526,7 +1522,7 @@ func (c *completer) unimportedPackages(ctx context.Context, seen map[string]stru
 		}
 		c.deepState.enqueue(candidate{
 			// Pass an empty *types.Package to disable deep completions.
-			obj:   types.NewPkgName(0, nil, pkg.GetTypes().Name(), types.NewPackage(path, pkg.Name())),
+			obj:   types.NewPkgName(0, nil, pkg.GetTypes().Name(), types.NewPackage(path, string(pkg.Name()))),
 			score: unimportedScore(relevances[path]),
 			imp:   imp,
 		})
@@ -1573,9 +1569,9 @@ func (c *completer) unimportedPackages(ctx context.Context, seen map[string]stru
 }
 
 // alreadyImports reports whether f has an import with the specified path.
-func alreadyImports(f *ast.File, path string) bool {
+func alreadyImports(f *ast.File, path source.ImportPath) bool {
 	for _, s := range f.Imports {
-		if source.ImportPath(s) == path {
+		if source.UnquoteImportPath(s) == path {
 			return true
 		}
 	}
@@ -2031,7 +2027,7 @@ Nodes:
 					//
 					// TODO: remove this after https://go.dev/issue/52503
 					info := &types.Info{Types: make(map[ast.Expr]types.TypeAndValue)}
-					types.CheckExpr(c.snapshot.FileSet(), c.pkg.GetTypes(), node.Fun.Pos(), node.Fun, info)
+					types.CheckExpr(c.pkg.FileSet(), c.pkg.GetTypes(), node.Fun.Pos(), node.Fun, info)
 					sig, _ = info.Types[node.Fun].Type.(*types.Signature)
 				}
 
