@@ -24,13 +24,35 @@ import (
 	"golang.org/x/tools/gopls/internal/govulncheck"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/internal/event/label"
+	"golang.org/x/tools/internal/event/tag"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/imports"
+	"golang.org/x/tools/internal/packagesinternal"
 )
 
+// A GlobalSnapshotID uniquely identifies a snapshot within this process and
+// increases monotonically with snapshot creation time.
+//
+// We use a distinct integral type for global IDs to help enforce correct
+// usage.
+type GlobalSnapshotID uint64
+
 // Snapshot represents the current state for the given view.
 type Snapshot interface {
-	ID() uint64
+	// SequenceID is the sequence id of this snapshot within its containing
+	// view.
+	//
+	// Relative to their view sequence ids are monotonically increasing, but this
+	// does not hold globally: when new views are created their initial snapshot
+	// has sequence ID 0. For operations that span multiple views, use global
+	// IDs.
+	SequenceID() uint64
+
+	// GlobalID is a globally unique identifier for this snapshot. Global IDs are
+	// monotonic: subsequent snapshots will have higher global ID, though
+	// subsequent snapshots in a view may not have adjacent global IDs.
+	GlobalID() GlobalSnapshotID
 
 	// View returns the View associated with this snapshot.
 	View() View
@@ -40,6 +62,11 @@ type Snapshot interface {
 	BackgroundContext() context.Context
 
 	// Fileset returns the Fileset used to parse all the Go files in this snapshot.
+	//
+	// If the files are known to belong to a specific Package, use
+	// Package.FileSet instead. (We plan to eliminate the
+	// Snapshot's cache of parsed files, and thus the need for a
+	// snapshot-wide FileSet.)
 	FileSet() *token.FileSet
 
 	// ValidBuildConfiguration returns true if there is some error in the
@@ -84,7 +111,7 @@ type Snapshot interface {
 	DiagnosePackage(ctx context.Context, pkg Package) (map[span.URI][]*Diagnostic, error)
 
 	// Analyze runs the analyses for the given package at this snapshot.
-	Analyze(ctx context.Context, pkgID string, analyzers []*Analyzer) ([]*Diagnostic, error)
+	Analyze(ctx context.Context, pkgID PackageID, analyzers []*Analyzer) ([]*Diagnostic, error)
 
 	// RunGoCommandPiped runs the given `go` command, writing its output
 	// to stdout and stderr. Verb, Args, and WorkingDir must be specified.
@@ -121,6 +148,10 @@ type Snapshot interface {
 	// the given go.mod file.
 	ModTidy(ctx context.Context, pm *ParsedModule) (*TidiedModule, error)
 
+	// ModVuln returns import vulnerability analysis for the given go.mod URI.
+	// Concurrent requests are combined into a single command.
+	ModVuln(ctx context.Context, modURI span.URI) (*govulncheck.Result, error)
+
 	// GoModForFile returns the URI of the go.mod file for the given URI.
 	GoModForFile(uri span.URI) span.URI
 
@@ -149,15 +180,18 @@ type Snapshot interface {
 
 	// GetActiveReverseDeps returns the active files belonging to the reverse
 	// dependencies of this file's package, checked in TypecheckWorkspace mode.
-	GetReverseDependencies(ctx context.Context, id string) ([]Package, error)
+	GetReverseDependencies(ctx context.Context, id PackageID) ([]Package, error)
 
 	// CachedImportPaths returns all the imported packages loaded in this
 	// snapshot, indexed by their package path (not import path, despite the name)
 	// and checked in TypecheckWorkspace mode.
-	CachedImportPaths(ctx context.Context) (map[string]Package, error)
+	CachedImportPaths(ctx context.Context) (map[PackagePath]Package, error)
 
-	// KnownPackages returns all the packages loaded in this snapshot, checked
-	// in TypecheckWorkspace mode.
+	// KnownPackages returns a new unordered list of all packages
+	// loaded in this snapshot, checked in TypecheckWorkspace mode.
+	//
+	// TODO(adonovan): opt: rewrite 'implementations' to avoid the
+	// need ever to "load everything at once" using this function.
 	KnownPackages(ctx context.Context) ([]Package, error)
 
 	// ActivePackages returns the packages considered 'active' in the workspace.
@@ -167,19 +201,21 @@ type Snapshot interface {
 	ActivePackages(ctx context.Context) ([]Package, error)
 
 	// AllValidMetadata returns all valid metadata loaded for the snapshot.
-	AllValidMetadata(ctx context.Context) ([]Metadata, error)
+	AllValidMetadata(ctx context.Context) ([]*Metadata, error)
 
 	// WorkspacePackageByID returns the workspace package with id, type checked
 	// in 'workspace' mode.
-	WorkspacePackageByID(ctx context.Context, id string) (Package, error)
+	WorkspacePackageByID(ctx context.Context, id PackageID) (Package, error)
 
 	// Symbols returns all symbols in the snapshot.
 	Symbols(ctx context.Context) map[span.URI][]Symbol
 
-	// Metadata returns package metadata associated with the given file URI.
-	MetadataForFile(ctx context.Context, uri span.URI) ([]Metadata, error)
+	// Metadata returns metadata for each package associated with the given file URI.
+	MetadataForFile(ctx context.Context, uri span.URI) ([]*Metadata, error)
 
 	// GetCriticalError returns any critical errors in the workspace.
+	//
+	// A nil result may mean success, or context cancellation.
 	GetCriticalError(ctx context.Context) *CriticalError
 
 	// BuildGoplsMod generates a go.mod file for all modules in the workspace.
@@ -187,6 +223,12 @@ type Snapshot interface {
 	BuildGoplsMod(ctx context.Context) (*modfile.File, error)
 }
 
+// SnapshotLabels returns a new slice of labels that should be used for events
+// related to a snapshot.
+func SnapshotLabels(snapshot Snapshot) []label.Label {
+	return []label.Label{tag.Snapshot.Of(snapshot.SequenceID()), tag.Directory.Of(snapshot.View().Folder())}
+}
+
 // PackageFilter sets how a package is filtered out from a set of packages
 // containing a given file.
 type PackageFilter int
@@ -243,18 +285,9 @@ type View interface {
 	// Folder returns the folder with which this view was created.
 	Folder() span.URI
 
-	// Shutdown closes this view, and detaches it from its session.
-	Shutdown(ctx context.Context)
-
 	// Options returns a copy of the Options for this view.
 	Options() *Options
 
-	// SetOptions sets the options of this view to new values.
-	// Calling this may cause the view to be invalidated and a replacement view
-	// added to the session. If so the new view will be returned, otherwise the
-	// original one will be.
-	SetOptions(context.Context, *Options) (View, error)
-
 	// Snapshot returns the current snapshot for the view, and a
 	// release function that must be called when the Snapshot is
 	// no longer needed.
@@ -278,17 +311,21 @@ type View interface {
 	// Vulnerabilites returns known vulnerabilities for the given modfile.
 	// TODO(suzmue): replace command.Vuln with a different type, maybe
 	// https://pkg.go.dev/golang.org/x/vuln/cmd/govulncheck/govulnchecklib#Summary?
-	Vulnerabilities(modfile span.URI) []govulncheck.Vuln
+	Vulnerabilities(modfile ...span.URI) map[span.URI]*govulncheck.Result
 
 	// SetVulnerabilities resets the list of vulnerabilites that exists for the given modules
 	// required by modfile.
-	SetVulnerabilities(modfile span.URI, vulnerabilities []govulncheck.Vuln)
+	SetVulnerabilities(modfile span.URI, vulncheckResult *govulncheck.Result)
 
 	// FileKind returns the type of a file
 	FileKind(FileHandle) FileKind
 
 	// GoVersion returns the configured Go version for this view.
 	GoVersion() int
+
+	// GoVersionString returns the go version string configured for this view.
+	// Unlike [GoVersion], this encodes the minor version and commit hash information.
+	GoVersionString() string
 }
 
 // A FileSource maps uris to FileHandles. This abstraction exists both for
@@ -338,20 +375,56 @@ type TidiedModule struct {
 }
 
 // Metadata represents package metadata retrieved from go/packages.
+type Metadata struct {
+	ID              PackageID
+	PkgPath         PackagePath
+	Name            PackageName
+	GoFiles         []span.URI
+	CompiledGoFiles []span.URI
+	ForTest         PackagePath // package path under test, or ""
+	TypesSizes      types.Sizes
+	Errors          []packages.Error
+	DepsByImpPath   map[ImportPath]PackageID  // may contain dups; empty ID => missing
+	DepsByPkgPath   map[PackagePath]PackageID // values are unique and non-empty
+	Module          *packages.Module
+	DepsErrors      []*packagesinternal.PackageError
+
+	// Config is the *packages.Config associated with the loaded package.
+	Config *packages.Config
+}
+
+// IsIntermediateTestVariant reports whether the given package is an
+// intermediate test variant, e.g. "net/http [net/url.test]".
 //
-// TODO(rfindley): move the strongly typed strings from the cache package here.
-type Metadata interface {
-	// PackageID is the unique package id.
-	PackageID() string
-
-	// PackageName is the package name.
-	PackageName() string
-
-	// PackagePath is the package path.
-	PackagePath() string
-
-	// ModuleInfo returns the go/packages module information for the given package.
-	ModuleInfo() *packages.Module
+// Such test variants arise when an x_test package (in this case net/url_test)
+// imports a package (in this case net/http) that itself imports the the
+// non-x_test package (in this case net/url).
+//
+// This is done so that the forward transitive closure of net/url_test has
+// only one package for the "net/url" import.
+// The intermediate test variant exists to hold the test variant import:
+//
+// net/url_test [net/url.test]
+//
+//	| "net/http" -> net/http [net/url.test]
+//	| "net/url" -> net/url [net/url.test]
+//	| ...
+//
+// net/http [net/url.test]
+//
+//	| "net/url" -> net/url [net/url.test]
+//	| ...
+//
+// This restriction propagates throughout the import graph of net/http: for
+// every package imported by net/http that imports net/url, there must be an
+// intermediate test variant that instead imports "net/url [net/url.test]".
+//
+// As one can see from the example of net/url and net/http, intermediate test
+// variants can result in many additional packages that are essentially (but
+// not quite) identical. For this reason, we filter these variants wherever
+// possible.
+func (m *Metadata) IsIntermediateTestVariant() bool {
+	return m.ForTest != "" && m.ForTest != m.PkgPath && m.ForTest+"_test" != m.PkgPath
 }
 
 var ErrViewExists = errors.New("view already exists for session")
@@ -593,27 +666,43 @@ func (a Analyzer) IsEnabled(view View) bool {
 	return a.Enabled
 }
 
-// Package represents a Go package that has been type-checked. It maintains
-// only the relevant fields of a *go/packages.Package.
+// Declare explicit types for package paths, names, and IDs to ensure that we
+// never use an ID where a path belongs, and vice versa. If we confused these,
+// it would result in confusing errors because package IDs often look like
+// package paths.
+type (
+	PackageID   string // go list's unique identifier for a package (e.g. "vendor/example.com/foo [vendor/example.com/bar.test]")
+	PackagePath string // name used to prefix linker symbols (e.g. "vendor/example.com/foo")
+	PackageName string // identifier in 'package' declaration (e.g. "foo")
+	ImportPath  string // path that appears in an import declaration (e.g. "example.com/foo")
+)
+
+// Package represents a Go package that has been parsed and type-checked.
+// It maintains only the relevant fields of a *go/packages.Package.
 type Package interface {
-	ID() string      // logically a cache.PackageID
-	Name() string    // logically a cache.PackageName
-	PkgPath() string // logically a cache.PackagePath
-	CompiledGoFiles() []*ParsedGoFile
-	File(uri span.URI) (*ParsedGoFile, error)
-	GetSyntax() []*ast.File
-	GetTypes() *types.Package
-	GetTypesInfo() *types.Info
+	// Metadata:
+	ID() PackageID
+	Name() PackageName
+	PkgPath() PackagePath
 	GetTypesSizes() types.Sizes
 	ForTest() string
-	DirectDep(packagePath string) (Package, error)        // logically a cache.PackagePath
-	ResolveImportPath(importPath string) (Package, error) // logically a cache.ImportPath
-	MissingDependencies() []string                        // unordered; logically cache.ImportPaths
-	Imports() []Package
-	Version() *module.Version
-	HasListOrParseErrors() bool
-	HasTypeErrors() bool
+	Version() *module.Version // may differ from Metadata.Module.Version
+
+	// Results of parsing:
+	FileSet() *token.FileSet
 	ParseMode() ParseMode
+	CompiledGoFiles() []*ParsedGoFile // (borrowed)
+	File(uri span.URI) (*ParsedGoFile, error)
+	GetSyntax() []*ast.File // (borrowed)
+	HasListOrParseErrors() bool
+
+	// Results of type checking:
+	GetTypes() *types.Package
+	GetTypesInfo() *types.Info
+	DirectDep(path PackagePath) (Package, error)
+	ResolveImportPath(path ImportPath) (Package, error)
+	Imports() []Package // new slice of all direct dependencies, unordered
+	HasTypeErrors() bool
 }
 
 // A CriticalError is a workspace-wide error that generally prevents gopls from
